"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var Color_1 = require("./Color");
var options_1 = require("./options");
var errors_1 = require("./errors");
var generateRandom = function (min, max) {
    return Math.floor((Math.random() * (max - min)) + min);
};
var colorDifference = function (color1, color2) {
    return (Math.abs(color1.r - color2.r) +
        Math.abs(color1.g - color2.g) +
        Math.abs(color1.b - color2.b));
};
var checkSimilarity = function (list, color, difference, considerations) {
    for (var i = 0; i < considerations; i++) {
        if (list[i] !== undefined && colorDifference(color, list[i]) < difference) {
            return true;
        }
    }
    return false;
};
var checkBrightness = function (color, brightness) {
    var getBrightness = function (color) { return ((color.r * 299) + (color.g * 587) + (color.b * 114)) / 1000; };
    if (brightness === null)
        return true;
    else if (brightness)
        return getBrightness(color) >= 123;
    else
        return getBrightness(color) <= 122;
};
var saveColor = function (list, color) {
    // rearrange previous colors
    for (var i = list.length; i > 0; i--)
        list[i] = list[i - 1];
    list[0] = color;
};
var randomColor = function () {
    // save previous colors to check similarity
    var previousColors = [];
    // closure
    return function (options) {
        var _a = options_1.parseOptions(options), difference = _a.difference, considerations = _a.considerations, brightness = _a.brightness;
        var color;
        var loopCount = 0;
        // do until a dissimilar color is found
        do {
            var r = generateRandom(30, 225);
            var g = generateRandom(30, 225);
            var b = generateRandom(30, 225);
            color = new Color_1.default(r, g, b);
            // throw error if color is not found while too many loops
            if (loopCount > 100)
                throw new errors_1.TooManyLoopsError();
            loopCount += 1;
        } while (checkSimilarity(previousColors, color, difference, considerations) || !checkBrightness(color, brightness));
        saveColor(previousColors, color);
        return color.toString();
    };
};
exports.default = randomColor;
